package consume

import (
	"applet/app/db"
	"applet/app/db/model"
	"applet/app/utils"
	"applet/app/utils/logx"
	"applet/consume/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/streadway/amqp"
	"time"
	"xorm.io/xorm"
)

func ZhiosGuideStoreOrder(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(1)
	delivery := ch.Consume(queue.Name, false)

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			//fmt.Println(string(res.Body))
			fmt.Println(">>>>>>>>>>>>>>>>CanalOrderConsume<<<<<<<<<<<<<<<<<<<<<<<<<")
			err = handleZhiosGuideStoreOrderSuccess(res.Body)
			//_ = res.Reject(false)
			if err == nil {
				_ = res.Ack(true)
			}
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func handleZhiosGuideStoreOrderSuccess(msg []byte) error {
	//1、解析canal采集至mq中queue的数据结构体
	var canalMsg *md.ZhiosGuideStoreOrder
	fmt.Println(string(msg))
	var tmpString string
	err := json.Unmarshal(msg, &tmpString)
	if err != nil {
		fmt.Println("===with", err.Error())
		return err
	}
	fmt.Println(tmpString)
	err = json.Unmarshal([]byte(tmpString), &canalMsg)
	if err != nil {
		fmt.Println("===with", err.Error())
		return err
	}
	mid := canalMsg.Mid
	eg := db.DBs[mid]
	if eg == nil {
		return nil
	}
	if canalMsg.Type == "success" {
		err := success(eg, canalMsg)
		if err != nil {
			return err
		}
	}
	if canalMsg.Type == "fail" {
		err := fail(eg, canalMsg)
		if err != nil {
			return err
		}
	}
	return nil
}
func success(eg *xorm.Engine, canalMsg *md.ZhiosGuideStoreOrder) error {
	sess := eg.NewSession()
	defer sess.Close()
	sess.Begin()
	var data model.GuideStoreOrder
	get, _ := sess.Where("pvd_oid=?", canalMsg.Oid).Get(&data)
	var newOrd model.OrdList
	sess.Where("pvd_oid=?", canalMsg.Oid).Get(&newOrd)
	if get == false {
		deductPrice := canalMsg.DeductPrice
		var storeOrder = model.GuideStoreOrder{
			Uid:            newOrd.Uid,
			Time:           time.Unix(int64(newOrd.CreateAt), 0),
			StoreId:        utils.StrToInt(canalMsg.StoreId),
			Gid:            newOrd.ItemId,
			State:          newOrd.State,
			Title:          canalMsg.ItemTitle,
			Pvd:            canalMsg.Pvd,
			SubsidyPrice:   canalMsg.SubsidyPrice,
			Fee:            canalMsg.Fee,
			DeductPrice:    deductPrice,
			OldDeductPrice: deductPrice,
			Price:          utils.Float64ToStr(newOrd.PaidPrice),
			Oid:            utils.Int64ToStr(newOrd.OrdId),
			PvdOid:         newOrd.PvdOid,
			DeductState:    1,
			DeductAt:       time.Now(),
		}
		if utils.StrToInt(canalMsg.State) == 4 {
			storeOrder.DeductState = 0
		}
		_, err := sess.Insert(&storeOrder)
		if err == nil && storeOrder.DeductState == 1 {
			coinId := db.SysCfgGetWithDb(eg, canalMsg.Mid, "guide_store_coin_id")
			err := UpdateUserFinValidAndInterFlowFreeze(sess,
				utils.AnyToString(deductPrice), "商品("+newOrd.ItemId+")下单扣除冻结积分", "0", 2, utils.StrToInt(canalMsg.StoreId), utils.StrToInt(coinId), 117, newOrd.OrdId)
			if err != nil {
				sess.Rollback()
				return err
			}
		}
		newOrd.OrderFormType = -2
		sess.Where("ord_id=?", newOrd.OrdId).Cols("order_form_type").Update(&newOrd)
	}
	sess.Commit()
	return nil
}

func fail(eg *xorm.Engine, canalMsg *md.ZhiosGuideStoreOrder) error {
	sess := eg.NewSession()
	defer sess.Close()
	sess.Begin()
	//m, _ := db.FreeProductByID(eg, canalMsg.ItemId, canalMsg.Pvd)
	//直接退回账号了 定制
	storeOrder := db.GuideStoreOrderSess(sess, utils.StrToInt64(canalMsg.Oid))
	//if m != nil {
	if storeOrder != nil && utils.StrToFloat64(storeOrder.DeductPrice) > 0 && storeOrder.DeductState == 1 {
		coinId := db.SysCfgGetWithDb(eg, canalMsg.Mid, "guide_store_coin_id")
		err := UpdateUserVirtualCoinFinValidAndInterFlow(sess,
			utils.AnyToString(storeOrder.DeductPrice), "商品("+storeOrder.Gid+")退款退回", "0", 1, storeOrder.StoreId, utils.StrToInt(coinId), 114, utils.StrToInt64(canalMsg.Oid))
		if err != nil {
			sess.Rollback()
			return err
		}
		storeOrder.DeductPrice = "0"
		storeOrder.DeductState = 2
		storeOrder.State = 4
		_, err = sess.Where("id=?", storeOrder.Id).Update(storeOrder)
		if err != nil {
			sess.Rollback()
			return err
		}
	}
	//}
	sess.Commit()
	return nil
}
func UpdateUserVirtualCoinFinValidAndInterFlow(sess *xorm.Session, money, Title, fee string, types, uid, coinId, transferType int, ordId int64) error {
	UserVirtualAmount, err := db.UserVirtualAmountFindById(sess, uid, coinId)
	if err != nil || UserVirtualAmount == nil {
		if err == nil {
			err = errors.New("获取用户余额信息失败")
		}
		return err
	}
	beforeAmount := UserVirtualAmount.Amount
	if types == 2 {
		UserVirtualAmount.Amount = utils.AnyToString(utils.AnyToFloat64(UserVirtualAmount.Amount) - utils.StrToFloat64(money))
		UserVirtualAmount.UseAmount = utils.AnyToString(utils.AnyToFloat64(UserVirtualAmount.UseAmount) + utils.StrToFloat64(money))
	} else if types == 1 {
		UserVirtualAmount.Amount = utils.AnyToString(utils.AnyToFloat64(UserVirtualAmount.Amount) + utils.StrToFloat64(money))
		UserVirtualAmount.UseAmount = utils.AnyToString(utils.AnyToFloat64(UserVirtualAmount.UseAmount) - utils.StrToFloat64(money))
	}
	afterAmount := UserVirtualAmount.Amount
	affected, err := db.UserVirtualAmountUpdate(sess, uid, coinId, UserVirtualAmount, "amount")
	if err != nil || affected == 0 {
		if err == nil {
			err = errors.New("更新用户余额信息失败")
		}
		return err
	}
	err = virtualCoinFlowInsert(sess, uid, coinId, money, fee, ordId, Title, types, transferType, beforeAmount, afterAmount)
	if err != nil {
		return err
	}
	return nil
}

func UpdateUserFinValidAndInterFlowFreeze(sess *xorm.Session, money, Title, fee string, types, uid, coinId, transferType int, ordId int64) error {
	UserVirtualAmount, err := db.UserVirtualAmountFindById(sess, uid, coinId)
	if err != nil || UserVirtualAmount == nil {
		if err == nil {
			err = errors.New("获取用户余额信息失败")
		}
		return err
	}
	beforeAmount := UserVirtualAmount.Amount
	if types == 2 {
		UserVirtualAmount.FreezeAmount = utils.AnyToString(utils.AnyToFloat64(UserVirtualAmount.FreezeAmount) - utils.StrToFloat64(money))
		UserVirtualAmount.UseAmount = utils.AnyToString(utils.AnyToFloat64(UserVirtualAmount.UseAmount) + utils.StrToFloat64(money))
	} else if types == 1 {
		UserVirtualAmount.FreezeAmount = utils.AnyToString(utils.AnyToFloat64(UserVirtualAmount.FreezeAmount) + utils.StrToFloat64(money))
		UserVirtualAmount.UseAmount = utils.AnyToString(utils.AnyToFloat64(UserVirtualAmount.UseAmount) - utils.StrToFloat64(money))
	}
	afterAmount := UserVirtualAmount.Amount
	affected, err := db.UserVirtualAmountUpdate(sess, uid, coinId, UserVirtualAmount, "freeze_amount,use_amount")
	if err != nil || affected == 0 {
		if err == nil {
			err = errors.New("更新用户余额信息失败")
		}
		return err
	}
	err = virtualCoinFlowInsert(sess, uid, coinId, money, fee, ordId, Title, types, transferType, beforeAmount, afterAmount)
	if err != nil {
		return err
	}
	return nil
}
func virtualCoinFlowInsert(sess *xorm.Session, uid, coinId int, money, SysFee string, ordId int64, ItemTitle string, types, transferType int, beforeAmount string, afterAmount string) error {
	now := time.Now()
	if err := db.UserVirtualCoinFlowInsertOne(
		sess,
		&model.UserVirtualCoinFlow{
			Uid:          uid,
			OrdId:        utils.Int64ToStr(ordId),
			CoinId:       coinId,
			Direction:    types,
			Title:        ItemTitle,
			Amout:        money,
			BeforeAmout:  beforeAmount,
			AfterAmout:   afterAmount,
			SysFee:       SysFee,
			CreateTime:   now,
			TransferType: transferType,
		}); err != nil {
		_ = logx.Warn(err)
		return err
	}
	return nil
}
