package consume

import (
	"applet/app/db"
	"applet/app/db/model"
	md2 "applet/app/md"
	"applet/app/utils"
	"applet/app/utils/logx"
	"applet/consume/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_mq.git/rabbit"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/lib/comm_plan"
	md3 "code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/md"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/rule"
	"code.fnuoos.com/go_rely_warehouse/zyos_go_order_relate_rule.git/svc"
	"encoding/json"
	"errors"
	"fmt"
	"github.com/jinzhu/copier"
	"github.com/streadway/amqp"
	"github.com/tidwall/gjson"
	"time"
	"xorm.io/xorm"
)

func ZhiosOrderBuckle(queue md.MqQueue) {
	fmt.Println(">>>>>>>>>>>>>>>>>>>>>>>>")
	ch, err := rabbit.Cfg.Pool.GetChannel()
	if err != nil {
		logx.Error(err)
		return
	}
	defer ch.Release()
	//1、将自己绑定到交换机上
	ch.Bind(queue.Name, queue.ExchangeName, queue.RoutKey)
	//2、取出数据进行消费
	ch.Qos(1)
	delivery := ch.Consume(queue.Name, false)

	var res amqp.Delivery
	var ok bool
	for {
		res, ok = <-delivery
		if ok == true {
			//fmt.Println(string(res.Body))
			fmt.Println(">>>>>>>>>>>>>>>>CanalOrderConsume<<<<<<<<<<<<<<<<<<<<<<<<<")
			err = handleZhiosOrderBuckle(res.Body)
			//_ = res.Reject(false)
			if err != nil {
				_ = res.Reject(false)
				//TODO::重新推回队列末尾，避免造成队列堵塞
				var msg *md.ZhiosOrderBuckle
				var tmpString string
				err := json.Unmarshal(res.Body, &tmpString)
				if err != nil {
					return
				}
				fmt.Println(tmpString)
				err = json.Unmarshal([]byte(tmpString), &msg)
				if err != nil {
					return
				}
				ch.Publish(queue.ExchangeName, utils.SerializeStr(msg), queue.RoutKey)
			} else {
				_ = res.Ack(true)
			}
		} else {
			panic(errors.New("error getting message"))
		}
	}
	fmt.Println("get msg done")
}

func handleZhiosOrderBuckle(msg []byte) error {
	//1、解析canal采集至mq中queue的数据结构体
	var canalMsg *md.ZhiosOrderBuckle
	fmt.Println(string(msg))
	var tmpString string
	err := json.Unmarshal(msg, &tmpString)
	if err != nil {
		fmt.Println("===with", err.Error())
		return err
	}
	fmt.Println(tmpString)
	err = json.Unmarshal([]byte(tmpString), &canalMsg)
	if err != nil {
		fmt.Println("===with", err.Error())
		return err
	}
	mid := canalMsg.Mid
	eg := db.DBs[mid]
	if eg == nil {
		return nil
	}
	sess := eg.NewSession()
	defer sess.Close()
	sess.Begin()

	ordData, err := db.OrderListByPvdOid(sess, canalMsg.Oid)
	if err != nil || ordData == nil {
		return nil
	}
	if ordData.OrderFormType < 0 || ordData.State > 3 {
		sess.Commit()

		return nil
	}
	user, _ := db.UserFindByID(eg, ordData.Uid)
	if user == nil {
		sess.Commit()

		return nil
	}
	//判断是否符合条件 TODO
	bools := checkUser(eg, canalMsg.Mid, user, utils.FloatFormat(ordData.BenefitAll, 3))
	if bools == false {
		sess.Commit()

		return nil
	}

	fmt.Println(ordData)
	//加入分佣关系链
	opts, commissionOpts, _ := svc.GetAllPlan(eg, canalMsg.Mid)
	if opts == nil {
		sess.Commit()

		return nil
	}
	BenefitAll := ordData.BenefitAll
	pvd := ordData.Pvd
	if ordData.OrderFormType == 1 { //京东自营
		pvd = md3.PVD_JDOwn
	}
	var rmd = md3.CommissionParam{IsTikTokTeamOrder: utils.IntToStr(ordData.IsTikTokTeamOrder)}
	opt, err := svc.GetPlanCfg(eg, pvd, canalMsg.Mid, opts, commissionOpts, map[int]string{}, &rmd)
	if err != nil {
		sess.Commit()
		return nil
	}

	if opt.BuckleRate == 0 {
		sess.Commit()

		return nil
	}
	//计算出暗扣的比例 TODO
	if opt.BuckleRate > 0 {
		BenefitAll = utils.FloatFormat(ordData.BenefitAll*(1-opt.BuckleRate), 9)
	}
	var ord = md2.OrderInfo{
		PvdOid:     ordData.PvdOid,
		Pvd:        pvd,
		ItemId:     ordData.ItemId,
		ItemNum:    ordData.ItemNum,
		ItemPrice:  ordData.ItemPrice,
		PaidPrice:  ordData.PaidPrice,
		OrderType:  ordData.OrderType,
		Commission: BenefitAll,
		State:      ordData.State,
	}
	//调用公共的分佣
	isShare := 0
	if ord.OrderType == 1 {
		isShare = 1
	}
	//计算每个用户的分佣
	req := md2.CommissionFirstParam{
		CommissionParam: md2.CommissionParam{
			Commission:        utils.Float64ToStr(ord.Commission),
			PaidPrice:         utils.Float64ToStr(ord.PaidPrice),
			OldPrice:          utils.Float64ToStr(ord.PaidPrice),
			IsTikTokTeamOrder: utils.IntToStr(ordData.IsTikTokTeamOrder),
		},
		Uid:              utils.IntToStr(ordData.Uid),
		IsShare:          isShare,
		Provider:         ord.Pvd,
		IsAllLevelReturn: 1,
	}
	if utils.InArr(ord.Pvd, []string{md3.PVD_TB, md3.PVD_TM, md3.PVD_PDD, md3.PVD_SN, md3.PVD_KL, md3.PVD_JD, md3.PVD_JDOwn, md3.PVD_VIP}) == false {
		req.CommissionParam.GoodsPrice = utils.Float64ToStr(ord.PaidPrice)
	}
	commissionList, err := GetCommissionByCommApi(eg, canalMsg.Mid, req)
	if err != nil || commissionList.LvUser == nil {
		sess.Commit()
		return nil
	}
	pvdFee := commissionList.PvdFee
	sysFee := commissionList.SysFee
	subsidyFee := commissionList.SubsidyFee
	lvUser := commissionList.LvUser
	profit := commissionList.Profit
	ordData.SubsidyFee = subsidyFee
	ordData.PvdCommission = pvdFee
	ordData.SysCommission = sysFee
	ordData.BuckleCommission = utils.Float64ToStr(BenefitAll)
	if ordData.OrderFormType == 2 {
		ordData.OrderFormType = -2
	} else {
		ordData.OrderFormType = -1

	}
	var selfRate float64 = 0
	var subsidyRate float64 = 0
	//处理记录佣金
	if opt != nil {
		profit = lvUser.Profit
		//判断下这个等级有么有设置
		ordData.UserCommission = profit
		ordData.SubsidyRate = subsidyRate
		ordData.UserCommissionRate = selfRate
		ordData.PlanCommissionId = opt.PlanCommissionId
		ordData.BenefitList = SerializeLvUser(NewCalcLvUserFee(lvUser))
	}
	has, _ := db.OrderListByUpdateOrd(sess, ordData)
	if has == false {
		sess.Rollback()
		return nil
	}

	//批量写入
	if lvUser != nil {
		err := OrderRelateInsert(eg, sess, ordData.OrdId, ordData.Pvd, ordData.CreateAt, lvUser, ordData, canalMsg.Mid, true, opt.Mode, "1")
		if err != nil {
			sess.Rollback()
			return err
		}
	}
	sess.Commit()
	return nil
}

func checkUser(eg *xorm.Engine, dbName string, user *model.User, commission float64) bool {
	buckleData := db.SysCfgGetWithDb(eg, dbName, "buckle_data")
	//注册时间
	lastMonthEstimateCommission := gjson.Get(buckleData, "last_month_estimate_commission").String() //用户上个月预估佣金金额大于
	monthOrderCount := gjson.Get(buckleData, "month_order_count").String()                          //用户每月订单笔数
	orderCommissionEnd := gjson.Get(buckleData, "order_commission_end").String()                    //用户订单佣金金额:结束区间
	orderCommissionStart := gjson.Get(buckleData, "order_commission_start").String()                //用户订单佣金金额:开始区间
	userRegDay := gjson.Get(buckleData, "user_reg_day").String()                                    //用户注册时间
	if (time.Now().Unix()-user.CreateAt.Unix())/86400 < utils.StrToInt64(userRegDay) && utils.StrToInt64(userRegDay) > 0 {
		fmt.Println("时间=============", (time.Now().Unix()-user.CreateAt.Unix())/86400)
		return false
	}
	if utils.StrToFloat64(lastMonthEstimateCommission) > 0 {
		lastMonthRange := utils.GetTimeRange("last_month")
		// 统计上月预估 （预估包含结算部分且是有效订单）
		sqlTpl2 := `SELECT cast(SUM(IF(ol.state<>4 or pco.state=1 or dol.id>0 or ro.status<>'已退款' or pso.status<>'订单退款', LEFT(olr.amount,LENGTH(olr.amount)-2), 0)) as decimal(50,4)) AS amount
FROM ord_list_relate olr
         LEFT JOIN ord_list ol ON olr.oid = ol.ord_id
		LEFT JOIN privilege_card_ord pco ON olr.oid =pco.ord_id
		LEFT JOIN duoyou_ord_list dol ON olr.oid =dol.oid
LEFT JOIN recharge_order ro ON olr.oid =ro.oid
		LEFT JOIN playlet_sale_order pso ON olr.oid =pso.custom_oid
WHERE olr.uid = ?
  AND olr.create_at >= ?
  AND olr.create_at < ?
`
		lastMonthResult, _ := db.QueryNativeString(eg, sqlTpl2, user.Uid, lastMonthRange["start"], lastMonthRange["end"])
		if len(lastMonthRange) == 0 {
			return false
		}
		fmt.Println("上月预估佣金=============", lastMonthResult[0]["amount"])

		if utils.StrToFloat64(lastMonthResult[0]["amount"]) < utils.StrToFloat64(lastMonthEstimateCommission) {
			return false
		}
	}
	if utils.StrToFloat64(monthOrderCount) > 0 {
		currentMonth := utils.GetTimeRange("current_month")
		count, _ := eg.Where("uid=? and create_at>=?", user.Uid, currentMonth["start"]).In("state", []string{"0", "1", "2", "3", "5"}).Count(&model.OrdList{})
		fmt.Println("本月订单数=============", count)

		if count < utils.StrToInt64(monthOrderCount) {
			return false
		}
	}
	if utils.StrToFloat64(orderCommissionEnd) > 0 && utils.StrToFloat64(orderCommissionStart) > 0 {
		fmt.Println("订单佣金=============", commission)
		if commission < utils.StrToFloat64(orderCommissionStart) {
			return false
		}
		if commission > utils.StrToFloat64(orderCommissionEnd) {
			return false
		}
	}
	return true
}

func OrderRelateInsert(eg *xorm.Engine, sess *xorm.Session, oid int64, pvd string, createTime int, lvUser *comm_plan.LvUser, newOrd *model.OrdList, masterId string, isDelete bool, mode string, isNew string) error {
	if lvUser == nil {
		return nil
	}
	uid := lvUser.Uid
	if uid == 0 {
		return nil
	}
	oldLvUser := lvUser
	oldLevel := 0
	byOid, _ := db.OrderListByOid(sess, utils.Int64ToStr(oid))
	data := OrderRelateInsertComm(eg, oid, pvd, createTime, lvUser, newOrd, masterId, mode)
	fmt.Println(data)

	if data == nil || len(data) == 0 {
		return nil
	}
	list, _ := db.OrderRelateFindByOid(sess, oid, pvd)
	listMap := make(map[int]model.OrdListRelate)
	if list != nil {
		for _, v := range *list {
			listMap[v.Uid] = v
		}
	}
	for k, v := range data {
		tmp, ok := listMap[v.Uid]
		if ok {
			data[k].OldAmount = tmp.Amount
			leave := utils.FloatFormat(tmp.Amount-v.Amount, 6)
			if leave >= 0 {
				var tmps = model.BuckleRecord{
					Date:         time.Now().Format("2006-01"),
					Time:         time.Now(),
					Uid:          v.Uid,
					OrdId:        utils.Int64ToStr(v.Oid),
					PvdOrdId:     byOid.PvdOid,
					Amount:       utils.Float64ToStrByPrec(leave, 6),
					BeforeAmount: utils.Float64ToStrByPrec(tmp.Amount, 6),
					AfterAmount:  utils.Float64ToStrByPrec(v.Amount, 6),
					Level:        v.Level,
				}
				_, err := sess.Insert(&tmps)
				if err != nil {
					return err
				}
			}
		}

	}

	fmt.Println(isDelete)
	_, err2 := db.OrderRelateDeleteByOid(sess, oid, pvd)
	if err2 != nil {
		return err2
	}
	_, err2 = db.VirtualCoinOrderRelateDeleteByOid(sess, oid, pvd)
	if err2 != nil {
		return err2
	}
	//后写入
	err := db.DbInsertBatchSess(sess, data)
	if err != nil {
		return err

	} else if lvUser.ProfitList != nil {
		// 插入虚拟币数据
		vcrData := CombineVirtualCoinRelateData(oldLvUser, oid, pvd, oldLevel, mode)
		if len(vcrData) == 0 {
			return nil
		}
		err := db.DbInsertBatchSess(sess, vcrData)
		if err != nil {
			return err

		}
	}
	if lvUser.TikTokOwnSubsidyFeeList != nil {
		_, err2 := db.TikTokTeamOrderRelateDeleteByOid(sess, oid, pvd)
		if err != nil {
			return err2
		}
		var teamData []model.TikTokTeamOrderRelate
		TikTokTeamCommission := newOrd.TikTokTeamCommission
		if utils.StrToFloat64(newOrd.TikTokTeamRealCommission) > 0 {
			TikTokTeamCommission = newOrd.TikTokTeamRealCommission
		}
		for k, v := range lvUser.TikTokOwnSubsidyFeeList {
			tmp := model.TikTokTeamOrderRelate{
				CoinId:     utils.StrToInt(k),
				Uid:        uid,
				Amount:     utils.Float64ToStrByPrec(v, 8),
				Oid:        utils.Int64ToStr(oid),
				Time:       time.Now(),
				Commission: TikTokTeamCommission,
				Pvd:        pvd,
			}
			teamData = append(teamData, tmp)
		}
		if len(teamData) == 0 {
			return nil
		}
		err := db.DbInsertBatchSess(sess, &teamData)
		if err != nil {
			return err

		}
	}
	return nil
}
func CombineVirtualCoinRelateData(lvUser *comm_plan.LvUser, oid int64, pvd string, level int, mode string) []*model.VirtualCoinRelate {
	var data []*model.VirtualCoinRelate
	//可能没有极差返利  只有补贴
	profitList := lvUser.ProfitList
	if len(profitList) == 0 {
		profitList = lvUser.SubsidyFeeList
	}
	if profitList != nil {
		var subsidyFeeList map[string]float64
		if lvUser.SubsidyFeeList != nil && len(lvUser.ProfitList) > 0 {
			subsidyFeeList = convertList2Map(lvUser.SubsidyFeeList)
		}
		var coinList = make([]string, 0)
		for _, v := range profitList {
			coinList = append(coinList, v.Cid)
		}
		if utils.InArr(mode, []string{"lv_winery", "public_platoon"}) && lvUser.SubsidyFeeList != nil { //补贴类型 没有的要补上
			subsidyFeeList = convertList2Map(lvUser.SubsidyFeeList)
			for _, v := range lvUser.SubsidyFeeList {
				if utils.InArr(v.Cid, coinList) == false && v.Val > 0 {
					v.Val = 0
					profitList = append(profitList, v)
				}
			}
		}
		for _, item := range profitList {
			if item.Cid != "0" && item.Cid != "commission" {
				//加上补贴
				subsidyFee := subsidyFeeList[item.Cid]
				var ownSubsidyFee float64 = 0
				ownSubsidyFee1, ok := lvUser.OwnSubsidyFeeList[item.Cid]
				if ok {
					ownSubsidyFee = ownSubsidyFee1
				}
				var additionalSubsidy float64 = 0
				profit := utils.Float64ToStrByPrec(item.Val+subsidyFee+ownSubsidyFee, 9)
				if utils.InArr(mode, []string{"lv_winery", "public_platoon"}) {
					profit = utils.Float64ToStrByPrec(item.Val+ownSubsidyFee, 9)
					additionalSubsidy = subsidyFee

				}
				if mode == "public_platoon" && level > 1 {
					profit = "0"
				}
				if lvUser.Uid == 0 {
					continue
				}
				var virtualCoinRelate = &model.VirtualCoinRelate{
					Oid:               oid,
					Uid:               lvUser.Uid,
					CoinId:            utils.StrToInt(item.Cid),
					Amount:            profit,
					Pvd:               pvd,
					CreateAt:          int(time.Now().Unix()),
					Level:             level,
					Mode:              mode,
					AdditionalSubsidy: utils.Float64ToStrByPrec(additionalSubsidy, 6),
					ExtendType:        lvUser.ExtendType,
				}
				data = append(data, virtualCoinRelate)
			}
		}
	}
	if lvUser.ParentUser != nil {
		level += 1
		data = append(data, CombineVirtualCoinRelateData(lvUser.ParentUser, oid, pvd, level, mode)...)
	}
	return data
}
func convertList2Map(a []*comm_plan.VirtualCoinCommission) (b map[string]float64) {
	b = make(map[string]float64)
	for _, i := range a {
		b[i.Cid] = i.Val
	}
	return b
}

func OrderRelateInsertComm(eg *xorm.Engine, oid int64, pvd string, createTime int, lvUser *comm_plan.LvUser, newOrd *model.OrdList, masterId string, mode string) []*model.OrdListRelate {
	var datas []*model.OrdListRelate
	if lvUser == nil {
		return datas
	}
	level := 0
	fmt.Println("关系链---"+utils.Int64ToStr(oid)+"---"+pvd+"---", utils.SerializeStr(lvUser))
	var ownSubsidyFee float64 = 0
	ownSubsidyFee1, ok := lvUser.OwnSubsidyFeeList["0"]
	if ok {
		ownSubsidyFee = ownSubsidyFee1
	}
	profit := utils.FloatFormat(lvUser.Profit+lvUser.SubsidyFee+ownSubsidyFee, 9)
	data := []*model.OrdListRelate{{Oid: oid, Uid: lvUser.Uid, Amount: profit, AdditionalSubsidy: utils.Float64ToStrByPrec(lvUser.AdditionalSubsidy, 9), Pvd: pvd, CreateAt: createTime, Level: level, Mode: mode, ExtendType: lvUser.ExtendType}}
	for lvUser.ParentUser != nil {
		lvUser = lvUser.ParentUser
		fmt.Println(lvUser)
		level = level + 1
		profit = utils.FloatFormat(lvUser.Profit+lvUser.SubsidyFee, 9)
		var additionalSubsidy float64 = 0
		if utils.InArr(mode, []string{"lv_winery", "public_platoon"}) {
			profit = utils.FloatFormat(lvUser.Profit, 6)
			additionalSubsidy = lvUser.SubsidyFee
		}
		if lvUser.Uid == 0 {
			continue
		}
		data = append(data, &model.OrdListRelate{
			Oid:               oid,
			Uid:               lvUser.Uid,
			Amount:            profit,
			Pvd:               pvd,
			CreateAt:          createTime,
			Level:             level,
			Mode:              mode,
			AdditionalSubsidy: utils.Float64ToStrByPrec(additionalSubsidy, 9),
			ExtendType:        lvUser.ExtendType,
		})
	}
	return data
}

func SerializeLvUser(lvUser map[int]float64) string {
	s := ""
	for k, v := range lvUser {
		s += "|" + utils.IntToStr(k) + ":" + utils.Float64ToStr(v)
	}
	return s
}

func NewCalcLvUserFee(lvUser *comm_plan.LvUser) map[int]float64 {
	fee := map[int]float64{}
	if lvUser == nil {
		return fee
	}
	if lvUser.ParentUser == nil {
		fee[lvUser.Uid] = lvUser.Profit
	}
	for lvUser.ParentUser != nil {
		fee[lvUser.Uid] = lvUser.Profit
		lvUser = lvUser.ParentUser
	}
	return fee
}

func SettleComm(eg *xorm.Engine, masterId string, CommissionParam md2.CommissionFirstParam) (float64, float64, float64, float64, *comm_plan.LvUser, error) {
	var CommissionParam1 = md3.CommissionFirstParam{}
	copier.Copy(&CommissionParam1, CommissionParam)
	profit, pvdFee, sysFee, subsidyFee, lvUser, err := rule.SettleCommWithOrder(eg, masterId, CommissionParam1)
	return profit, pvdFee, sysFee, subsidyFee, lvUser, err
}

// GetCommissionByCommApi 调用公共接口获取佣金
func GetCommissionByCommApi(eg *xorm.Engine, masterId string, req md2.CommissionFirstParam) (*comm_plan.SettleCommissionToGuide, error) {
	profit, pvdFee, sysFee, subsidyFee, lvUser, err := SettleComm(eg, masterId, req)
	if err != nil {
		return nil, err
	}
	result := &comm_plan.SettleCommissionToGuide{
		Profit:     profit,
		PvdFee:     pvdFee,
		SysFee:     sysFee,
		SubsidyFee: subsidyFee,
		LvUser:     lvUser,
	}
	return result, nil
}
